include(CMakeFindDependencyMacro)

list(APPEND CMAKE_MODULE_PATH "${CMAKE_CURRENT_LIST_DIR}")

include(${CMAKE_CURRENT_LIST_DIR}/AluminumConfigVersion.cmake)
set(ALUMINUM_VERSION ${PACKAGE_VERSION})

set(AL_HAS_CUDA )
set(AL_HAS_ROCM )
set(AL_HAS_MPI_CUDA )
set(AL_HAS_HOST_TRANSFER )
set(AL_HAS_NCCL )

set(MPI_CXX_COMPILER "/usr/local/mpi/openmpi/bin/mpicxx" CACHE FILEPATH
  "The MPI CXX compiler wrapper.")
find_package(MPI 3.0 REQUIRED COMPONENTS CXX)

find_dependency(HWLOC)

if (AL_HAS_ROCM)
  find_package(HIP REQUIRED)
  if (AL_HAS_NCCL)
    find_package(rccl REQUIRED)
  endif (AL_HAS_NCCL)
elseif (AL_HAS_CUDA)
  find_package(CUDA REQUIRED)

  if (NOT TARGET cuda::cuda)
    add_library(cuda::cuda INTERFACE IMPORTED)
  endif ()

  set_property(TARGET cuda::cuda
    PROPERTY INTERFACE_LINK_LIBRARIES ${CUDA_LIBRARIES})

  set_property(TARGET cuda::cuda
    PROPERTY INTERFACE_INCLUDE_DIRECTORIES ${CUDA_INCLUDE_DIRS})

  if (AL_HAS_NCCL)
    find_package(NCCL 2.0.0 QUIET)
    if (NOT NCCL_FOUND)
      if (NOT NCCL_DIR)
        set(NCCL_DIR "")
      endif()
      if (NOT NCCL_DIR AND NOT NCCL_LIBRARY AND NOT NCCL_INCLUDE_PATH)
        set(NCCL_LIBRARY "" CACHE FILEPATH
          "The NCCL library" FORCE)
        set(NCCL_INCLUDE_PATH "" CACHE PATH
          "The NCCL header include directory" FORCE)
      endif ()
      find_package(NCCL 2.0.0 REQUIRED)
    endif (NOT NCCL_FOUND)

    set_property(TARGET cuda::cuda APPEND
      PROPERTY INTERFACE_LINK_LIBRARIES cuda::nccl)
  endif ()
endif (AL_HAS_ROCM)


####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was AluminumConfig.cmake.in                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../../" ABSOLUTE)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

####################################################################################

set(_TMP_INCLUDE_DIRS "${PACKAGE_PREFIX_DIR}/include")
foreach (_DIR ${_TMP_INCLUDE_DIRS})
  set_and_check(_INCLUDE_DIR "${_DIR}")
  list(APPEND ALUMINUM_INCLUDE_DIRS "${_INCLUDE_DIR}")
endforeach (_DIR "${_TMP_INCLUDE_DIRS}")

set(_TMP_LIBRARY_DIRS "${PACKAGE_PREFIX_DIR}/lib")
foreach (_DIR ${_TMP_LIBRARY_DIRS})
  set_and_check(_LIBRARY_DIR "${_DIR}")
  list(APPEND ALUMINUM_LIBRARY_DIRS "${_LIBRARY_DIR}")
endforeach (_DIR ${_TMP_LIBRARY_DIRS})

if (NOT TARGET AL::aluminum)
  include(${CMAKE_CURRENT_LIST_DIR}/AluminumTargets.cmake)
endif ()

check_required_components(Aluminum)

set(Aluminum_LIBRARIES AL::Al)
